/*
 * Copyright (C) 2021, Xilinx Inc - All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#pragma once

#include <map>
#include <functional>
#include "xmacpp/DecoderSession.h"
#include "SessionBase.h"
#include "InternalFrame.h"

namespace xma {
//////////////////////////////////////////////////////////////////////////////
#define XMA_DEFAULT_PKT_POOL_SIZE 128
#define XMA_DEFAULT_PKT_SIZE 4096

class InternalDecoderSession final : virtual public DecoderSession, virtual public SessionBase {
public:
  InternalDecoderSession(IntSessionFactoryPtr sf, XmaDecoderProperties* props);
  virtual ~InternalDecoderSession();

  virtual FramePropertiesPtr GetProps() const override;

  virtual int32_t SetLog(LogPtr log) override;

  virtual int32_t Enqueue(queue_t& data) override;
  virtual int32_t Dequeue(queue_t& data) override;

private:
  IntFramePtr GetFrame();

protected:
  using lock_t           = std::unique_lock<std::mutex>;
  using vpi_mem_handle_t = std::unique_ptr<void, std::function<void(VpiMemHandle mem_ctx)>>;
  using dec_handle_t     = std::unique_ptr<void, std::function<void(void*)>>;
  using pkt_handle_t     = std::unique_ptr<VpiPkt, std::function<void(VpiPkt*)>>;
  using pool_handle_t    = std::unique_ptr<void, std::function<void(void*)>>;
  using pic_handle_t     = std::unique_ptr<VpiFrm, std::function<void(VpiFrm*)>>;
  using buf_map_t        = std::map<DataBuffer*, DataBufferPtr>;
  using ts_map_t         = std::map<int64_t, int64_t>;
  mutable std::mutex m_mutex;
  vpi_mem_handle_t   m_mem_ctx;
  dec_handle_t       m_handle;
  FramePropertiesPtr m_props;
  IntFramePtr        m_first_frame;
  pkt_handle_t       m_pkt;
  pool_handle_t      m_pool;
  pic_handle_t       m_pic;
  int32_t            m_wait        = 0;
  bool               m_low_latency = false;
  bool               m_eos_pushed  = false;
  bool               m_eos_found   = false;
  bool               m_eos_pulled  = false;
  XmaFraction        m_framerate;
  buf_map_t          m_buffer_map;
  ts_map_t           m_timestamp_map;
  int64_t            m_frame_sent      = 0;
  int64_t            m_frame_recv      = 0;
  bool               m_latency_logging = false;
  struct timespec    m_latency;
  int64_t            m_time_taken;
};

using IntDecoderSessionPtr = std::shared_ptr<InternalDecoderSession>;

}; // Namespace xma
