/*
 * Copyright (C) 2021, Xilinx Inc - All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#ifndef _XLNX_TRANSCODER_CONSTANTS_H_
#define _XLNX_TRANSCODER_CONSTANTS_H_

#include <stdio.h>

#define XLNX_TRANSCODER_APP_MODULE "xlnx_transcoder_app"

#define TRANSCODE_APP_SUCCESS  0
#define TRANSCODE_APP_STOP     1
#define TRANSCODE_PARSING_DONE 2

#define TRANSCODE_APP_FAILURE (-1)

/* Decoder constants */
#define DEC_MAX_PARAMS          3
#define DEC_INPUT_EOF           1
#define DEC_MAX_OUT_BUFFERS     20
#define DEC_LOW_LATENCY_DEFAULT 0

/* Scaler constants */
#define SCAL_MAX_ABR_CHANNELS 8
#define SCAL_MAX_PARAMS       2
#define SCAL_RATE_STRING_LEN  8
#define SCAL_MAX_INPUT_WIDTH  3840
#define SCAL_MAX_INPUT_HEIGHT 2160
#define SCAL_MIN_INPUT_WIDTH  64
#define SCAL_MIN_INPUT_HEIGHT 64

/* Transcoder constants */
#define TRANSCODE_MAX_ABR_CHANNELS (SCAL_MAX_ABR_CHANNELS + 1)
#define MAX_ARG_SIZE               64

#define TRANSCODE_DEC_FLUSH_MODE  1
#define TRANSCODE_SCAL_FLUSH_MODE 2
#define TRANSCODE_LA_FLUSH_MODE   3
#define TRANSCODE_ENC_FLUSH_MODE  4

/* Encoder constants */
#define ENC_MIN_LOOKAHEAD_DEPTH     -1
#define ENC_MAX_LOOKAHEAD_DEPTH     53
#define ENC_DEFAULT_LOOKAHEAD_DEPTH ENC_MIN_LOOKAHEAD_DEPTH
#define ENC_SUPPORTED_MIN_WIDTH     64
#define ENC_SUPPORTED_MAX_WIDTH     3840
#define ENC_MAX_LA_INPUT_WIDTH      ENC_SUPPORTED_MAX_WIDTH
#define ENC_DEFAULT_WIDTH           1920
#define ENC_SUPPORTED_MIN_HEIGHT    64
#define ENC_SUPPORTED_MAX_HEIGHT    2160
#define ENC_MAX_LA_INPUT_HEIGHT     ENC_SUPPORTED_MAX_HEIGHT
#define ENC_DEFAULT_HEIGHT          1080
#define ENC_SUPPORTED_MIN_BITRATE   -1
#define ENC_SUPPORTED_MAX_BITRATE   35000000
#define ENC_DEFAULT_BITRATE         200
#define ENC_MIN_MAX_BITRATE         ENC_SUPPORTED_MIN_BITRATE
#define ENC_MAX_MAX_BITRATE         ENC_SUPPORTED_MAX_BITRATE
#define ENC_DEFAULT_MAX_BITRATE     ENC_MIN_MAX_BITRATE
#define ENC_DEFAULT_FRAMERATE       30
#define ENC_DEFAULT_GOP_SIZE        -1
#define ENC_SUPPORTED_MAX_QP        51
#define ENC_SUPPORTED_MIN_QP        -1
#define ENC_DEFAULT_QP              ENC_SUPPORTED_MIN_QP
#define ENC_DEFAULT_LEVEL           0
#define ENC_DEFAULT_SPAT_AQ_GAIN    50
#define ENC_MAX_SPAT_AQ_GAIN        100
#define ENC_MIN_NUM_B_FRAMES        -1
#define ENC_MAX_NUM_B_FRAMES        3
#define ENC_MAX_NUM_B_FRAMES_AV1    7
#define ENC_DEFAULT_NUM_B_FRAMES    ENC_MIN_NUM_B_FRAMES
#define ENC_MAX_OPTIONS_SIZE        2048
#define ENC_MAX_PARAMS              13
#define ENC_AQ_GAIN_NOT_USED        255
#define ENC_SUPPORTED_MIN_AQ_GAIN   0
#define ENC_SUPPORTED_MAX_AQ_GAIN   255
#define ENC_SPATIAL_AQ_GAIN_DEFAULT 255
#define ENC_SPATIAL_AQ_AUTO         -1
#define ENC_SPATIAL_AQ_DISABLE      0
#define ENC_SPATIAL_AQ_ENABLE       1
#define ENC_TEMPORAL_AQ_AUTO        -1
#define ENC_TEMPORAL_AQ_DISABLE     0
#define ENC_TEMPORAL_AQ_ENABLE      1
#define ENC_DEFAULT_SPATIAL_AQ      -1
#define ENC_DEFAULT_TEMPORAL_AQ     -1
#define ENC_MIN_TUNE_METRICS        1
#define ENC_MAX_TUNE_METRICS        4
#define ENC_DEFAULT_TUNE_METRICS    ENC_MIN_TUNE_METRICS
#define ENC_CRF_DISABLE             0
#define ENC_CRF_ENABLE              1
#define ENC_IDR_DISABLE             0
#define ENC_IDR_ENABLE              1
#define ENC_DYNAMIC_GOP_AUTO        -1
#define ENC_DYNAMIC_GOP_DISABLE     0
#define ENC_DYNAMIC_GOP_ENABLE      1
#define ENC_DYNAMIC_GOP_DEFAULT     ENC_DYNAMIC_GOP_AUTO

#define ENC_DEFAULT_PROFILE 0
#define ENC_MIN_MIN_QP      ENC_SUPPORTED_MIN_QP
#define ENC_MAX_MIN_QP      ENC_SUPPORTED_MAX_QP
#define ENC_DEFAULT_MIN_QP  -1
#define ENC_DEFAULT_MAX_QP  -1
#define ENC_MIN_MAX_QP      ENC_SUPPORTED_MIN_QP
#define ENC_MAX_MAX_QP      ENC_SUPPORTED_MAX_QP

#define ENC_MIN_GOP_SIZE        -1
#define ENC_MAX_GOP_SIZE        1000
#define ENC_DEFAULT_DEVICE_TYPE XMA_ENC_DEVICE_TYPE_1

#define ENC_SUPPORTED_MAX_PIXELS                                               \
    ((ENC_SUPPORTED_MAX_WIDTH) * (ENC_SUPPORTED_MAX_HEIGHT))

#define ENC_MAX_LA_PIXELS ((ENC_MAX_LA_INPUT_WIDTH) * (ENC_MAX_LA_INPUT_HEIGHT))

#define XLNX_ENC_LINE_ALIGN(x, LINE_SIZE)                                      \
    (((((size_t)x) + ((size_t)LINE_SIZE - 1)) & (~((size_t)LINE_SIZE - 1))))

#define TRANS_DEFAULT_LOG_LEVEL    XMA_ERROR_LOG
#define TRANS_DEFAULT_LOG_LOCATION XMA_LOG_TYPE_CONSOLE
#define TRANS_DEFAULT_LOG_FILE     (XLNX_TRANSCODER_APP_MODULE ".log")

#define ENC_PROFILE_AUTO -1
#define ENC_AV1_MAIN     200

/* Lookahead constants */
#define XLNX_SCLEVEL1 2

typedef enum {
    ENCODER_ID_H264 = XMA_H264_ENCODER_TYPE,
    ENCODER_ID_HEVC = XMA_HEVC_ENCODER_TYPE,
    ENCODER_ID_AV1  = XMA_AV1_ENCODER_TYPE
} XlnxEncoderCodecID;

typedef enum { DECODER_ID_H264 = 1, DECODER_ID_HEVC } XlnxDecoderCodecID;

/* Transcoder states */
typedef enum {

    TRANSCODE_DEC_READ_FRAME = 0,
    TRANSCODE_DEC_SEND_INPUT,
    TRANSCODE_DEC_SEND_LAST_FRAME,
    TRANSCODE_DEC_GET_OUTPUT,
    TRANSCODE_DEC_FLUSH,
    TRANSCODE_SCAL_PROCESS_FRAME,
    TRANSCODE_SCAL_FLUSH,
    TRANSCODE_LA_PROCESS,
    TRANSCODE_LA_FLUSH,
    TRANSCODE_ENC_PROCESS_FRAME,
    TRANSCODE_ENC_NULL_FRAME,
    TRANSCODE_ENC_FLUSH,
    TRANSCODE_EOF,
    TRANSCODE_STOP,
    TRANSCODE_DONE
} XlnxTranscodeStates;

/* Enum for transcoder command line arguments */
typedef enum {
    HELP_ARG = 0,
    NO_OF_STREAMS,
    TRANSCODE_DEVICE_ID_ARG,
    TRANSCODE_STREAM_LOOP_ARG,
    TRANSCODE_NUM_FRAMES_ARG,
    TRANSCODE_GENERIC_MAX_ARG,
    LOG_LEVEL_ARG,
    LOG_LOCATION_ARG,
    LOG_FILE_ARG
} XlnxTranscodeArgIdentifiers;

/* Enum for decoder command line arguments */
typedef enum {
    DEC_CODEC_ID_ARG = TRANSCODE_GENERIC_MAX_ARG,
    DEC_INPUT_FILE_ARG,
    DEC_LOW_LATENCY_ARG,
    DEC_LATENCY_LOGGING_ARG,
    DEC_PIX_FMT_ARG,
    DEC_MAX_ARG
} XlnxDecArgIdentifiers;

/* Enum for scaler command line arguments */
typedef enum {
    SCAL_FILTER_ARG = DEC_MAX_ARG,
    SCAL_ENABLE_PIPELINE_ARG,
    SCAL_NUM_OUTPUTS_ARG,
    SCAL_OUTPUT_WIDTH_ARG,
    SCAL_OUTPUT_HEIGHT_ARG,
    SCAL_OUTPUT_RATE_ARG,
    SCAL_LATENCY_LOGGING_ARG,
    SCAL_MAX_ARG
} XlnxScalerArgIdentifiers;

/* Enum for encoder command line arguments */
typedef enum XlnxEncArgIdentifiers {
    ENC_CODEC_ID_ARG = SCAL_MAX_ARG,
    ENC_DEVICE_TYPE_ARG,
    ENC_BITRATE_ARG,
    ENC_FPS_ARG,
    ENC_INTRA_PERIOD_ARG,
    ENC_MIN_QP_ARG,
    ENC_MAX_QP_ARG,
    ENC_SPAT_AQ_GAIN_ARG,
    ENC_TEMP_AQ_GAIN_ARG,
    ENC_SPAT_AQ_ARG,
    ENC_TEMP_AQ_ARG,
    ENC_QP_MODE_ARG,
    ENC_RC_MODE_ARG,
    ENC_CRF_ARG,
    ENC_MAX_BITRATE_ARG,
    ENC_FORCE_IDR_ARG,
    ENC_NUM_BFRAMES_ARG,
    ENC_IDR_PERIOD_ARG,
    ENC_PRESET_ARG,
    ENC_PROFILE_ARG,
    ENC_LEVEL_ARG,
    ENC_NUM_SLICES_ARG,
    ENC_LOOKAHEAD_DEPTH_ARG,
    ENC_QP_ARG,
    ENC_NUM_CORES_ARG,
    TIER_ARG,
    ENC_LATENCY_LOGGING_ARG,
    ENC_DYNAMIC_GOP_ARG,
    ENC_EXPERT_OPTIONS_ARG,
    ENC_TUNE_METRICS_ARG,
    ENC_OUTPUT_FILE_ARG
} XlnxEncArgIdentifiers;

/* Enum for lookahead arguments */
typedef enum XlnxLAExtParams {
    EParamIntraPeriod = 0,
    EParamLADepth,
    EParamEnableHwInBuf,
    EParamSpatialAQMode,
    EParamTemporalAQMode,
    EParamRateControlMode,
    EParamSpatialAQGain,
    EParamNumBFrames,
    EParamCodecType,
    EParamLatencyLogging
} XlnxLAExtParams;

/* H264 Encoder supported profiles */
typedef enum Xlnxh264Profiles {
    ENC_H264_BASELINE,
    ENC_H264_MAIN,
    ENC_H264_HIGH,
    ENC_H264_HIGH_10,
    ENC_H264_HIGH_10_INTRA
} Xlnxh264Profiles;

/* HEVC Encoder supported profiles */
typedef enum XlnxHevcProfiles {
    ENC_HEVC_MAIN = 100,
    ENC_HEVC_MAIN_INTRA,
    ENC_HEVC_MAIN_10,
    ENC_HEVC_MAIN10_INTRA
} XlnxHevcProfiles;

/* QP Mode supported values */
typedef enum XlnxQpModes {
    ENC_DEFAULT_QP_MODE = 0,
    ENC_QP_MODE_AUTO,
    ENC_QP_MODE_UNIFORM
} XlnxQpModes;

/*  RC Mode supported values */
typedef enum XlnxRcMode {
    ENC_RC_MODE_AUTO = -1,
    ENC_RC_MODE_CONSTANT_QP,
    ENC_RC_MODE_CBR,
    ENC_RC_MODE_VBR,
    ENC_RC_MODE_CVBR,
    ENC_RC_MODE_DEFAULT = ENC_RC_MODE_AUTO
} XlnxRcMode;

typedef enum XlnxTuneMetrics {
    ENC_TUNE_METRICS_VQ = 1,
    ENC_TUNE_METRICS_PSNR,
    ENC_TUNE_METRICS_SSIM,
    ENC_TUNE_METRICS_VMAF,
    ENC_TUNE_METRICS_DEFAULT = ENC_TUNE_METRICS_VQ
} XlnxTuneMetrics;

typedef enum XlnxEncTiers {
    ENC_TIER_AUTO = -1,
    ENC_TIER_MAIN,
    ENC_TIER_HIGH,
    ENC_TIER_DEFAULT = ENC_TIER_AUTO
} XlnxEncTiers;

typedef enum XlnxAv1DeviceTypes {
    ENC_AV1_DEVICE_TYPE_ANY = XMA_ENC_DEVICE_TYPE_ANY,
    ENC_AV1_DEVICE_TYPE_1,
    ENC_AV1_DEVICE_TYPE_2
} XlnxAv1DeviceTypes;

#endif //_XLNX_TRANSCODER_CONSTANTS_H_
